<?php

namespace Company\CompanyStructure\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Company\CompanyStructure\Http\Requests\DepartmentRequest;
use Company\CompanyStructure\Interfaces\DepartmentRepositoryInterface;
use Company\CompanyStructure\Models\Department;

/**
 * Resource controller class for department.
 */
class DepartmentResourceController extends BaseController
{

    /**
     * Initialize department resource controller.
     *
     * @param type DepartmentRepositoryInterface $department
     *
     * @return null
     */
    public function __construct(DepartmentRepositoryInterface $department)
    {
        parent::__construct();
        $this->repository = $department;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Company\CompanyStructure\Repositories\Criteria\DepartmentResourceCriteria::class);
    }

    /**
     * Display a list of department.
     *
     * @return Response
     */
    public function index(DepartmentRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Company\CompanyStructure\Repositories\Presenter\DepartmentPresenter::class)
                ->$function();
        }

        $departments = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('company_structure::department.names'))
            ->view('company_structure::department.index', true)
            ->data(compact('departments', 'view'))
            ->output();
    }

    /**
     * Display department.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return Response
     */
    public function show(DepartmentRequest $request, Department $department)
    {

        if ($department->exists) {
            $view = 'company_structure::department.show';
        } else {
            $view = 'company_structure::department.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('company_structure::department.name'))
            ->data(compact('department'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new department.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DepartmentRequest $request)
    {

        $department = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company_structure::department.name')) 
            ->view('company_structure::department.create', true) 
            ->data(compact('department'))
            ->output();
    }

    /**
     * Create new department.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DepartmentRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $department                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('company_structure::department.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company_structure/department/' . $department->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company_structure/department'))
                ->redirect();
        }

    }

    /**
     * Show department for editing.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return Response
     */
    public function edit(DepartmentRequest $request, Department $department)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company_structure::department.name'))
            ->view('company_structure::department.edit', true)
            ->data(compact('department'))
            ->output();
    }

    /**
     * Update the department.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return Response
     */
    public function update(DepartmentRequest $request, Department $department)
    {
        try {
            $attributes = $request->all();

            $department->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company_structure::department.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company_structure/department/' . $department->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company_structure/department/' . $department->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the department.
     *
     * @param Model   $department
     *
     * @return Response
     */
    public function destroy(DepartmentRequest $request, Department $department)
    {
        try {

            $department->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company_structure::department.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('company_structure/department/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company_structure/department/' . $department->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple department.
     *
     * @param Model   $department
     *
     * @return Response
     */
    public function delete(DepartmentRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company_structure::department.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('company_structure/department'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company_structure/department'))
                ->redirect();
        }

    }

    /**
     * Restore deleted departments.
     *
     * @param Model   $department
     *
     * @return Response
     */
    public function restore(DepartmentRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('company_structure::department.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/company_structure/department'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company_structure/department/'))
                ->redirect();
        }

    }

}
