<?php

namespace Company\CompanyStructure\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Company\CompanyStructure\Http\Requests\DepartmentRoleRequest;
use Company\CompanyStructure\Interfaces\DepartmentRoleRepositoryInterface;
use Company\CompanyStructure\Models\DepartmentRole;

/**
 * Resource controller class for department_role.
 */
class DepartmentRoleResourceController extends BaseController
{

    /**
     * Initialize department_role resource controller.
     *
     * @param type DepartmentRoleRepositoryInterface $department_role
     *
     * @return null
     */
    public function __construct(DepartmentRoleRepositoryInterface $department_role)
    {
        parent::__construct();
        $this->repository = $department_role;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Company\CompanyStructure\Repositories\Criteria\DepartmentRoleResourceCriteria::class);
    }

    /**
     * Display a list of department_role.
     *
     * @return Response
     */
    public function index(DepartmentRoleRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Company\CompanyStructure\Repositories\Presenter\DepartmentRolePresenter::class)
                ->$function();
        }

        $department_roles = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('company_structure::department_role.names'))
            ->view('company_structure::department_role.index', true)
            ->data(compact('department_roles', 'view'))
            ->output();
    }

    /**
     * Display department_role.
     *
     * @param Request $request
     * @param Model   $department_role
     *
     * @return Response
     */
    public function show(DepartmentRoleRequest $request, DepartmentRole $department_role)
    {

        if ($department_role->exists) {
            $view = 'company_structure::department_role.show';
        } else {
            $view = 'company_structure::department_role.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('company_structure::department_role.name'))
            ->data(compact('department_role'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new department_role.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DepartmentRoleRequest $request)
    {

        $department_role = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company_structure::department_role.name')) 
            ->view('company_structure::department_role.create', true) 
            ->data(compact('department_role'))
            ->output();
    }

    /**
     * Create new department_role.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DepartmentRoleRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $department_role                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('company_structure::department_role.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company_structure/department_role/' . $department_role->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company_structure/department_role'))
                ->redirect();
        }

    }

    /**
     * Show department_role for editing.
     *
     * @param Request $request
     * @param Model   $department_role
     *
     * @return Response
     */
    public function edit(DepartmentRoleRequest $request, DepartmentRole $department_role)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company_structure::department_role.name'))
            ->view('company_structure::department_role.edit', true)
            ->data(compact('department_role'))
            ->output();
    }

    /**
     * Update the department_role.
     *
     * @param Request $request
     * @param Model   $department_role
     *
     * @return Response
     */
    public function update(DepartmentRoleRequest $request, DepartmentRole $department_role)
    {
        try {
            $attributes = $request->all();

            $department_role->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company_structure::department_role.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company_structure/department_role/' . $department_role->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company_structure/department_role/' . $department_role->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the department_role.
     *
     * @param Model   $department_role
     *
     * @return Response
     */
    public function destroy(DepartmentRoleRequest $request, DepartmentRole $department_role)
    {
        try {

            $department_role->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company_structure::department_role.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('company_structure/department_role/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company_structure/department_role/' . $department_role->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple department_role.
     *
     * @param Model   $department_role
     *
     * @return Response
     */
    public function delete(DepartmentRoleRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company_structure::department_role.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('company_structure/department_role'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company_structure/department_role'))
                ->redirect();
        }

    }

    /**
     * Restore deleted department_roles.
     *
     * @param Model   $department_role
     *
     * @return Response
     */
    public function restore(DepartmentRoleRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('company_structure::department_role.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/company_structure/department_role'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company_structure/department_role/'))
                ->redirect();
        }

    }

}
