<?php

namespace Company\CompanyStructure\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Company\CompanyStructure\Http\Requests\LineRequest;
use Company\CompanyStructure\Interfaces\LineRepositoryInterface;
use Company\CompanyStructure\Models\Line;

/**
 * Resource controller class for line.
 */
class LineResourceController extends BaseController
{

    /**
     * Initialize line resource controller.
     *
     * @param type LineRepositoryInterface $line
     *
     * @return null
     */
    public function __construct(LineRepositoryInterface $line)
    {
        parent::__construct();
        $this->repository = $line;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Company\CompanyStructure\Repositories\Criteria\LineResourceCriteria::class);
    }

    /**
     * Display a list of line.
     *
     * @return Response
     */
    public function index(LineRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Company\CompanyStructure\Repositories\Presenter\LinePresenter::class)
                ->$function();
        }

        $lines = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('company_structure::line.names'))
            ->view('company_structure::line.index', true)
            ->data(compact('lines', 'view'))
            ->output();
    }

    /**
     * Display line.
     *
     * @param Request $request
     * @param Model   $line
     *
     * @return Response
     */
    public function show(LineRequest $request, Line $line)
    {

        if ($line->exists) {
            $view = 'company_structure::line.show';
        } else {
            $view = 'company_structure::line.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('company_structure::line.name'))
            ->data(compact('line'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new line.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LineRequest $request)
    {

        $line = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company_structure::line.name')) 
            ->view('company_structure::line.create', true) 
            ->data(compact('line'))
            ->output();
    }

    /**
     * Create new line.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LineRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $line                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('company_structure::line.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company_structure/line/' . $line->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company_structure/line'))
                ->redirect();
        }

    }

    /**
     * Show line for editing.
     *
     * @param Request $request
     * @param Model   $line
     *
     * @return Response
     */
    public function edit(LineRequest $request, Line $line)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company_structure::line.name'))
            ->view('company_structure::line.edit', true)
            ->data(compact('line'))
            ->output();
    }

    /**
     * Update the line.
     *
     * @param Request $request
     * @param Model   $line
     *
     * @return Response
     */
    public function update(LineRequest $request, Line $line)
    {
        try {
            $attributes = $request->all();

            $line->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company_structure::line.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company_structure/line/' . $line->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company_structure/line/' . $line->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the line.
     *
     * @param Model   $line
     *
     * @return Response
     */
    public function destroy(LineRequest $request, Line $line)
    {
        try {

            $line->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company_structure::line.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('company_structure/line/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company_structure/line/' . $line->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple line.
     *
     * @param Model   $line
     *
     * @return Response
     */
    public function delete(LineRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company_structure::line.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('company_structure/line'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company_structure/line'))
                ->redirect();
        }

    }

    /**
     * Restore deleted lines.
     *
     * @param Model   $line
     *
     * @return Response
     */
    public function restore(LineRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('company_structure::line.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/company_structure/line'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company_structure/line/'))
                ->redirect();
        }

    }

}
